@@ -34,6 +34,8 @@
 
 """Internal Torrent class"""
 
+from __future__ import with_statement
+
 import os
 import time
 from urllib import unquote
@@ -98,6 +100,14 @@ class TorrentOptions(dict):
         self["file_priorities"] = []
         self["mapped_files"] = {}
 
+
+class TorrentError(object):
+    def __init__(self, error_message, was_paused=False, restart_to_resume=False):
+        self.error_message = error_message
+        self.was_paused = was_paused
+        self.restart_to_resume = restart_to_resume
+
+
 class Torrent(object):
     """Torrent holds information about torrents added to the libtorrent session.
     """
@@ -141,6 +151,9 @@ class Torrent(object):
         # Store the magnet uri used to add this torrent if available
         self.magnet = magnet
 
+        # Torrent state e.g. Paused, Downloading, etc.
+        self.state = None
+
         # Holds status info so that we don't need to keep getting it from lt
         self.status = self.handle.status()
 
@@ -185,11 +198,14 @@ class Torrent(object):
         # Various torrent options
         self.handle.resolve_countries(True)
 
-        self.set_options(self.options)
+        # Details of torrent forced into error state (i.e. not by libtorrent).
+        self.forced_error = None
 
         # Status message holds error info about the torrent
         self.statusmsg = "OK"
 
+        self.set_options(self.options)
+
         # The torrents state
         self.update_state()
 
@@ -209,8 +225,6 @@ class Torrent(object):
         self.forcing_recheck = False
         self.forcing_recheck_paused = False
 
-        log.debug("Torrent object created.")
-
     ## Options methods ##
     def set_options(self, options):
         OPTIONS_FUNCS = {
@@ -235,6 +249,10 @@ class Torrent(object):
 
 
     def set_max_connections(self, max_connections):
+        if max_connections == 0:
+            max_connections = -1
+        elif max_connections == 1:
+            max_connections = 2
         self.options["max_connections"] = int(max_connections)
         self.handle.set_max_connections(max_connections)
 
@@ -292,19 +310,19 @@ class Torrent(object):
         self.options["move_completed_path"] = move_completed_path
 
     def set_file_priorities(self, file_priorities):
-        if len(file_priorities) != len(self.get_files()):
-            log.debug("file_priorities len != num_files")
-            self.options["file_priorities"] = self.handle.file_priorities()
-            return
-
-        if self.options["compact_allocation"]:
-            log.debug("setting file priority with compact allocation does not work!")
-            self.options["file_priorities"] = self.handle.file_priorities()
-            return
+        handle_file_priorities = self.handle.file_priorities()
+        # Workaround for libtorrent 1.1 changing default priorities from 1 to 4.
+        if 4 in handle_file_priorities:
+           handle_file_priorities = [1 if x == 4 else x for x in handle_file_priorities]
 
         log.debug("setting %s's file priorities: %s", self.torrent_id, file_priorities)
 
-        self.handle.prioritize_files(file_priorities)
+        if (self.handle.has_metadata() and not self.options["compact_allocation"] and
+                file_priorities and len(file_priorities) == len(self.get_files())):
+            self.handle.prioritize_files(file_priorities)
+        else:
+            log.debug("Unable to set new file priorities.")
+            file_priorities = handle_file_priorities
 
         if 0 in self.options["file_priorities"]:
             # We have previously marked a file 'Do Not Download'
@@ -316,14 +334,14 @@ class Torrent(object):
                     self.update_state()
                     break
 
-        self.options["file_priorities"] = self.handle.file_priorities()
+        self.options["file_priorities"] = handle_file_priorities
         if self.options["file_priorities"] != list(file_priorities):
             log.warning("File priorities were not set for this torrent")
 
         # Set the first/last priorities if needed
         self.set_prioritize_first_last(self.options["prioritize_first_last_pieces"])
 
-    def set_trackers(self, trackers):
+    def set_trackers(self, trackers, reannounce=True):
         """Sets trackers"""
         if trackers == None:
             trackers = []
@@ -353,7 +371,7 @@ class Torrent(object):
         #    log.debug("tier: %s tracker: %s", t["tier"], t["url"])
         # Set the tracker list in the torrent object
         self.trackers = trackers
-        if len(trackers) > 0:
+        if len(trackers) > 0 and reannounce:
             # Force a reannounce if there is at least 1 tracker
             self.force_reannounce()
 
@@ -366,51 +384,56 @@ class Torrent(object):
 
     def set_tracker_status(self, status):
         """Sets the tracker status"""
+        self.tracker_host = None
         self.tracker_status = self.get_tracker_host() + ": " + status
 
     def update_state(self):
         """Updates the state based on what libtorrent's state for the torrent is"""
         # Set the initial state based on the lt state
         LTSTATE = deluge.common.LT_TORRENT_STATE
-        ltstate = int(self.handle.status().state)
+        status = self.handle.status()
+        ltstate = status.state
 
-        # Set self.state to the ltstate right away just incase we don't hit some
-        # of the logic below
-        if ltstate in LTSTATE:
-            self.state = LTSTATE[ltstate]
-        else:
-            self.state = str(ltstate)
+        # Set self.state to the ltstate right away just incase we don't hit some of the logic below
+        old_state = self.state
+        self.state = LTSTATE.get(int(ltstate), str(ltstate))
 
-        log.debug("set_state_based_on_ltstate: %s", deluge.common.LT_TORRENT_STATE[ltstate])
-        log.debug("session.is_paused: %s", component.get("Core").session.is_paused())
+        is_paused = self.handle.is_paused()
+        is_auto_managed = self.handle.is_auto_managed()
+        session_paused = component.get("Core").session.is_paused()
 
         # First we check for an error from libtorrent, and set the state to that
         # if any occurred.
-        if len(self.handle.status().error) > 0:
+        if self.forced_error:
+            self.state = "Error"
+            self.set_status_message("Error: " + self.forced_error.error_message)
+        elif status.error:
             # This is an error'd torrent
             self.state = "Error"
-            self.set_status_message(self.handle.status().error)
-            if self.handle.is_paused():
+            self.set_status_message(status.error)
+            if is_paused:
                 self.handle.auto_managed(False)
-            return
-
-        if ltstate == LTSTATE["Queued"] or ltstate == LTSTATE["Checking"]:
-            if self.handle.is_paused():
+        else:
+            if is_paused and is_auto_managed and not session_paused:
+                self.state = "Queued"
+            elif is_paused or session_paused:
                 self.state = "Paused"
-            else:
+            elif ltstate == LTSTATE["Queued"] or ltstate == LTSTATE["Checking"] or \
+                    ltstate == LTSTATE["Checking Resume Data"]:
                 self.state = "Checking"
-            return
-        elif ltstate == LTSTATE["Downloading"] or ltstate == LTSTATE["Downloading Metadata"]:
-            self.state = "Downloading"
-        elif ltstate == LTSTATE["Finished"] or ltstate == LTSTATE["Seeding"]:
-            self.state = "Seeding"
-        elif ltstate == LTSTATE["Allocating"]:
-            self.state = "Allocating"
+            elif ltstate == LTSTATE["Downloading"] or ltstate == LTSTATE["Downloading Metadata"]:
+                self.state = "Downloading"
+            elif ltstate == LTSTATE["Finished"] or ltstate == LTSTATE["Seeding"]:
+                self.state = "Seeding"
+            elif ltstate == LTSTATE["Allocating"]:
+                self.state = "Allocating"
 
-        if self.handle.is_paused() and self.handle.is_auto_managed() and not component.get("Core").session.is_paused():
-            self.state = "Queued"
-        elif component.get("Core").session.is_paused() or (self.handle.is_paused() and not self.handle.is_auto_managed()):
-            self.state = "Paused"
+        if self.state != old_state:
+            log.debug("Using torrent state from lt: %s, auto_managed: %s, paused: %s, session_paused: %s",
+                      ltstate, is_auto_managed, is_paused, session_paused)
+            log.debug("Torrent %s set from %s to %s: '%s'",
+                      self.torrent_id, old_state, self.state, self.statusmsg)
+            component.get("EventManager").emit(TorrentStateChangedEvent(self.torrent_id, self.state))
 
     def set_state(self, state):
         """Accepts state strings, ie, "Paused", "Seeding", etc."""
@@ -424,6 +447,37 @@ class Torrent(object):
     def set_status_message(self, message):
         self.statusmsg = message
 
+    def force_error_state(self, message, restart_to_resume=True):
+        """Forces the torrent into an error state.
+
+        For setting an error state not covered by libtorrent.
+
+        Args:
+            message (str): The error status message.
+            restart_to_resume (bool, optional): Prevent resuming clearing the error, only restarting
+                session can resume.
+        """
+        status = self.handle.status()
+        self.handle.auto_managed(False)
+        self.forced_error = TorrentError(message, status.paused, restart_to_resume)
+        if not status.paused:
+            self.handle.pause()
+        self.update_state()
+
+    def clear_forced_error_state(self, update_state=True):
+        if not self.forced_error:
+            return
+
+        if self.forced_error.restart_to_resume:
+            log.error("Restart deluge to clear this torrent error")
+
+        if not self.forced_error.was_paused and self.options["auto_managed"]:
+            self.handle.auto_managed(True)
+        self.forced_error = None
+        self.set_status_message("OK")
+        if update_state:
+            self.update_state()
+
     def get_eta(self):
         """Returns the ETA in seconds for this torrent"""
         if self.status == None:
@@ -502,13 +556,15 @@ class Torrent(object):
             except UnicodeDecodeError:
                 client = str(peer.client).decode("latin-1")
 
-            # Make country a proper string
-            country = str()
-            for c in peer.country:
-                if not c.isalpha():
-                    country += " "
-                else:
-                    country += c
+            try:
+                country = component.get("Core").geoip_instance.country_code_by_addr(peer.ip[0])
+            except AttributeError:
+                country = peer.country
+
+            try:
+                country = "".join([char if char.isalpha() else " " for char in country])
+            except TypeError:
+                country = ""
 
             ret.append({
                 "client": client,
@@ -526,10 +582,21 @@ class Torrent(object):
         """Returns the torrents queue position"""
         return self.handle.queue_position()
 
+    def get_file_priorities(self):
+        """Return the file priorities"""
+        if not self.handle.has_metadata():
+            return []
+
+        if not self.options["file_priorities"]:
+            # Ensure file_priorities option is populated.
+            self.set_file_priorities([])
+
+        return self.options["file_priorities"]
+
     def get_file_progress(self):
         """Returns the file progress as a list of floats.. 0.0 -> 1.0"""
         if not self.handle.has_metadata():
-            return 0.0
+            return []
 
         file_progress = self.handle.file_progress()
         ret = []
@@ -538,6 +605,8 @@ class Torrent(object):
                 ret.append(float(file_progress[i]) / float(f["size"]))
             except ZeroDivisionError:
                 ret.append(0.0)
+            except IndexError:
+                return []
 
         return ret
 
@@ -620,7 +689,6 @@ class Torrent(object):
             "compact": self.options["compact_allocation"],
             "distributed_copies": distributed_copies,
             "download_payload_rate": self.status.download_payload_rate,
-            "file_priorities": self.options["file_priorities"],
             "hash": self.torrent_id,
             "is_auto_managed": self.options["auto_managed"],
             "is_finished": self.is_finished,
@@ -719,6 +787,7 @@ class Torrent(object):
         fns = {
             "comment": ti_comment,
             "eta": self.get_eta,
+            "file_priorities": self.get_file_priorities,
             "file_progress": self.get_file_progress,
             "files": self.get_files,
             "is_seed": self.handle.is_seed,
@@ -779,6 +848,8 @@ class Torrent(object):
 
     def pause(self):
         """Pause this torrent"""
+        if self.state == "Error":
+            return False
         # Turn off auto-management so the torrent will not be unpaused by lt queueing
         self.handle.auto_managed(False)
         if self.handle.is_paused():
@@ -802,7 +873,8 @@ class Torrent(object):
 
         if self.handle.is_paused() and self.handle.is_auto_managed():
             log.debug("Torrent is being auto-managed, cannot resume!")
-            return
+        elif self.forced_error and self.forced_error.was_paused:
+            log.debug("Skip resuming Error state torrent that was originally paused.")
         else:
             # Reset the status message just in case of resuming an Error'd torrent
             self.set_status_message("OK")
@@ -826,6 +898,11 @@ class Torrent(object):
 
             return True
 
+        if self.forced_error and not self.forced_error.restart_to_resume:
+            self.clear_forced_error_state()
+        elif self.state == "Error" and not self.forced_error:
+            self.handle.clear_error()
+
     def connect_peer(self, ip, port):
         """adds manual peer"""
         try:
@@ -838,27 +915,31 @@ class Torrent(object):
     def move_storage(self, dest):
         """Move a torrent's storage location"""
         try:
-           dest = unicode(dest, "utf-8")
+            dest = unicode(dest, "utf-8")
         except TypeError:
-           # String is already unicode
-           pass
+            # String is already unicode
+            pass
 
         if not os.path.exists(dest):
             try:
                 # Try to make the destination path if it doesn't exist
                 os.makedirs(dest)
-            except IOError, e:
-                log.exception(e)
-                log.error("Could not move storage for torrent %s since %s does not exist and could not create the directory.", self.torrent_id, dest_u)
+            except OSError, ex:
+                log.error("Could not move storage for torrent %s since %s does "
+                          "not exist and could not create the directory: %s",
+                          self.torrent_id, dest, ex)
                 return False
 
+        kwargs = {}
+        if deluge.common.VersionSplit(lt.version) >= deluge.common.VersionSplit("1.0.0.0"):
+            kwargs['flags'] = 2  # dont_replace
         dest_bytes = dest.encode('utf-8')
         try:
             # libtorrent needs unicode object if wstrings are enabled, utf8 bytestring otherwise
             try:
-                self.handle.move_storage(dest)
+                self.handle.move_storage(dest, **kwargs)
             except TypeError:
-                self.handle.move_storage(dest_bytes)
+                self.handle.move_storage(dest_bytes, **kwargs)
         except Exception, e:
             log.error("Error calling libtorrent move_storage: %s" % e)
             return False
@@ -868,8 +949,12 @@ class Torrent(object):
     def save_resume_data(self):
         """Signals libtorrent to build resume data for this torrent, it gets
         returned in a libtorrent alert"""
-        self.handle.save_resume_data()
-        self.waiting_on_resume_data = True
+        # Don't generate fastresume data if torrent is in a Deluge Error state.
+        if self.forced_error:
+            log.debug("Skipped creating resume_data while in Error state")
+        else:
+            self.handle.save_resume_data()
+            self.waiting_on_resume_data = True
 
     def on_metadata_received(self):
         if self.options["prioritize_first_last_pieces"]:
@@ -889,7 +974,8 @@ class Torrent(object):
             md = lt.bdecode(self.torrent_info.metadata())
             torrent_file = {}
             torrent_file["info"] = md
-            open(path, "wb").write(lt.bencode(torrent_file))
+            with open(path, "wb") as _file:
+                _file.write(lt.bencode(torrent_file))
         except Exception, e:
             log.warning("Unable to save torrent file: %s", e)
 
@@ -926,16 +1012,27 @@ class Torrent(object):
 
     def force_recheck(self):
         """Forces a recheck of the torrents pieces"""
-        paused = self.handle.is_paused()
+        self.forcing_recheck = True
+        if self.forced_error:
+            self.forcing_recheck_paused = self.forced_error.was_paused
+            self.clear_forced_error_state(update_state=False)
+        else:
+            self.forcing_recheck_paused = self.handle.is_paused()
+        # Store trackers for paused torrents to prevent unwanted announce before pausing again.
+        if self.forcing_recheck_paused:
+            self.set_trackers(None, reannounce=False)
+            self.handle.replace_trackers([])
+
         try:
             self.handle.force_recheck()
             self.handle.resume()
         except Exception, e:
             log.debug("Unable to force recheck: %s", e)
-            return False
-        self.forcing_recheck = True
-        self.forcing_recheck_paused = paused
-        return True
+            self.forcing_recheck = False
+            if self.forcing_recheck_paused:
+                self.set_trackers(torrent.trackers, reannounce=False)
+
+        return self.forcing_recheck
 
     def rename_files(self, filenames):
         """Renames files in the torrent. 'filenames' should be a list of
@@ -984,4 +1081,3 @@ class Torrent(object):
         for key in self.prev_status.keys():
             if not self.rpcserver.is_session_valid(key):
                 del self.prev_status[key]
-
