@@ -102,6 +102,7 @@ CONFIG_DEFAULTS = {
 
     # Server Settings
     "base": "/",
+    "interface": "0.0.0.0",
     "port": 8112,
     "https": False,
     "pkey": "ssl/daemon.pkey",
@@ -234,9 +235,10 @@ class Flag(resource.Resource):
             request.setHeader("cache-control",
                               "public, must-revalidate, max-age=86400")
             request.setHeader("content-type", "image/png")
-            data = open(filename, "rb")
+            with open(filename, "rb") as _file:
+                data = _file.read()
             request.setResponseCode(http.OK)
-            return data.read()
+            return data
         else:
             request.setResponseCode(http.NOT_FOUND)
             return ""
@@ -282,7 +284,9 @@ class LookupResource(resource.Resource, component.Component):
                 log.debug("Serving path: '%s'", path)
                 mime_type = mimetypes.guess_type(path)
                 request.setHeader("content-type", mime_type[0])
-                return compress(open(path, "rb").read(), request)
+                with open(path, "rb") as _file:
+                    data = _file.read()
+                return compress(data, request)
 
         request.setResponseCode(http.NOT_FOUND)
         return "<h1>404 - Not Found</h1>"
@@ -386,19 +390,20 @@ class ScriptResource(resource.Resource, component.Component):
 
                         order_file = os.path.join(dirpath, '.order')
                         if os.path.isfile(order_file):
-                            for line in open(order_file, 'rb'):
-                                line = line.strip()
-                                if not line or line[0] == '#':
-                                    continue
-                                try:
-                                    pos, filename = line.split()
-                                    files.pop(files.index(filename))
-                                    if pos == '+':
-                                        files.insert(0, filename)
-                                    else:
-                                        files.append(filename)
-                                except:
-                                    pass
+                            with open(order_file, 'rb') as _file:
+                                for line in _file:
+                                    line = line.strip()
+                                    if not line or line[0] == '#':
+                                        continue
+                                    try:
+                                        pos, filename = line.split()
+                                        files.pop(files.index(filename))
+                                        if pos == '+':
+                                            files.insert(0, filename)
+                                        else:
+                                            files.append(filename)
+                                    except:
+                                        pass
 
                         dirpath = dirpath[len(filepath)+1:]
                         if dirpath:
@@ -439,7 +444,9 @@ class ScriptResource(resource.Resource, component.Component):
                 log.debug("Serving path: '%s'", path)
                 mime_type = mimetypes.guess_type(path)
                 request.setHeader("content-type", mime_type[0])
-                return compress(open(path, "rb").read(), request)
+                with open(path, "rb") as _file:
+                    data = _file.read()
+                return compress(data, request)
 
         request.setResponseCode(http.NOT_FOUND)
         return "<h1>404 - Not Found</h1>"
@@ -617,6 +624,7 @@ class DelugeWeb(component.Component):
         self.socket = None
         self.top_level = TopLevel()
         self.site = server.Site(self.top_level)
+        self.interface = self.config["interface"]
         self.port = self.config["port"]
         self.https = self.config["https"]
         self.pkey = self.config["pkey"]
@@ -659,9 +667,8 @@ class DelugeWeb(component.Component):
             reactor.run()
 
     def start_normal(self):
-        self.socket = reactor.listenTCP(self.port, self.site)
-        log.info("serving on %s:%s view at http://127.0.0.1:%s", "0.0.0.0",
-            self.port, self.port)
+        self.socket = reactor.listenTCP(self.port, self.site, interface=self.interface)
+        log.info("Serving on %s:%s view at http://%s:%s", self.interface, self.port, self.interface, self.port)
 
     def start_ssl(self):
         log.debug("Enabling SSL with PKey: %s, Cert: %s", self.pkey, self.cert)
@@ -674,8 +681,8 @@ class DelugeWeb(component.Component):
         options = CertificateOptions(privateKey=private_key, certificate=certificate, method=SSL.SSLv23_METHOD)
         options.getContext().set_options(SSL.OP_NO_SSLv2 | SSL.OP_NO_SSLv3)
 
-        self.socket = reactor.listenSSL(self.port, self.site, options)
-        log.info("Serving on %s:%s view at https://127.0.0.1:%s", "0.0.0.0", self.port, self.port)
+        self.socket = reactor.listenSSL(self.port, self.site, options, interface=self.interface)
+        log.info("Serving on %s:%s view at https://%s:%s", self.interface, self.port, self.interface, self.port)
 
     def stop(self):
         log.info("Shutting down webserver")
